module LAB10_1_TOP(
  output audio_mclk, // master clock
  output audio_lrck, // left-right clock
  output audio_sck,  // serial clock
  output audio_sdin, // serial audio data input
  
  input  clk,        // clock from the crystal
  input  rst_n       // active low reset
);
  
  //////////////////////////////////////
  // to play the 14 sounds repeatedly //
  //////////////////////////////////////
  
  reg [26:0] seconds_counter; // seconds_counter[26] is 1-second clock
  
  always @(posedge clk or negedge rst_n) begin
    if (~rst_n) seconds_counter <= 'b0;
    else        seconds_counter <= (seconds_counter < 27'd99_999_999)
                                 ? (seconds_counter + 'b1)
                                 : ('b0);
  end
  
  reg [3:0] note_counter; // counts from 0 to 13
  
  always @(negedge seconds_counter[26] or negedge rst_n) begin
    if (~rst_n) note_counter <= 'b0;
    else        note_counter <= (note_counter < 4'd13)
                              ? (note_counter + 'b1)
                              : ('b0);
  end
  
  ////////////////////////////
  // look-up note_frequenct //
  ////////////////////////////
  
  reg [9:0] note_frequency;
  
  always @* begin
    case (note_counter)
      4'h0: note_frequency <= 10'd261; // Do
      4'h1: note_frequency <= 10'd293; // Re
      4'h2: note_frequency <= 10'd330; // Mi
      4'h3: note_frequency <= 10'd349; // Fa
      4'h4: note_frequency <= 10'd392; // So
      4'h5: note_frequency <= 10'd440; // La
      4'h6: note_frequency <= 10'd494; // Si
      4'h7: note_frequency <= 10'd524; // Do
      4'h8: note_frequency <= 10'd588; // Re
      4'h9: note_frequency <= 10'd660; // Mi
      4'hA: note_frequency <= 10'd698; // Fa
      4'hB: note_frequency <= 10'd784; // So
      4'hC: note_frequency <= 10'd880; // La
      4'hD: note_frequency <= 10'd988; // Si
      default: note_frequency <= 'b0;
    endcase
  end
  
  //////////////////////
  // audio generation //
  //////////////////////
  
  wire [15:0] audio_left;
  wire [15:0] audio_right;
  
  // a general frequency divider 
  // to generate the required frequencies for speaker clock
  note_gen Ung(
    .audio_left,  // left sound audio
    .audio_right, // right sound audio
    .note_div(27'd100_000_000 / 2'd2 / note_frequency),
    .clk,         // clock from crystal
    .rst_n        // active low reset
  );
  
  ////////////////////////
  // Speaker controllor //
  ////////////////////////
  
  // a stereo signal parallel-to-serial processor
  // to generate the speaker control signals
  speaker_control Usc(
    // outputs
    .audio_mclk,  // master clock
    .audio_lrck,  // left-right clock
    .audio_sck,   // serial clock
    .audio_sdin,  // serial audio data input
    // inputs
    .audio_left,  // left channel audio data input
    .audio_right, // right channel audio data input
    .clk,         // clock from the crystal
    .rst_n        // active low reset
  );
  
endmodule
